﻿Type=Class
Version=3.5
@EndOfDesignText@
#Region Library Attributes
	#Event: Click
	#Event: LongClick
#End Region

'Class Name: ModularMenu
'Author: Giancarlo "Dominex" Fioretti
'Version: 1.37
'B4A Version Used: 3.50
'Required Libraries: AnimationPlus 2
'Last Modified: 07/05/2014
'-------------------------
'Class module
Private Sub Class_Globals
	Private vActivity As Activity
	Private vModule As Object
	Private pnlSfondo,pnlMenu,pnlShade(3*Density) As Panel
	Private MenuList As ListView
	Private vMenuLock,AnimationInProgress,MenuIsShow As Boolean
	Private vMaxItemShow,WidthMenu,nShade,AnimDuration,MaxWidthMenu,vHeightItem As Int
	Private ColorBorder,ColorBackground,ColorText As Int
	Private Dist = 10dip,Corner = 3dip,EdgeIcon = 3dip As Float
	Private ListItem As List
	Private vEventName As String
End Sub

'Initializes the ModularMenu.
'MaxItemShow - maximum number of item visible in the menu without scrolling.
'HeightItem - height of each item.
'Parent - where is the Activity create the ModularMenu.
'Module - must be Me
'
'Example:<code>
'Menu.Initialize(7,43dip,Activity,Me)</code>
Public Sub Initialize (MaxItemShow As Int,HeightItem As Int,Parent As Activity,Module As Object)
	'--OPTION SETUP-----------
	AnimDuration = 400
	MaxWidthMenu = 270dip
	ColorBorder = Colors.Gray
	ColorBackground = Colors.DarkGray
	ColorText = Colors.White
	'-------------------------
	vHeightItem = HeightItem
	vMaxItemShow = MaxItemShow
	vActivity = Parent
	vModule = Module
	nShade = pnlShade.Length-1
	WidthMenu = Min(100%x-Dist*2-nShade*2,MaxWidthMenu)
	ListItem.Initialize
End Sub

'Returns True if the menu is open.
Public Sub IsOpen As Boolean
	Return MenuIsShow
End Sub

'Passing in a list of the items stored in the menu.
'See the example Multi-Activity.
Public Sub SaveItem As List
	Return ListItem
End Sub

'Upload the menu with the items on the List previously saved.
'See the example Multi-Activity.
Public Sub LoadItem(MenuItem As List) As Boolean
	If MenuItem.IsInitialized = False Then Return False
	ListItem.Clear
	ListItem.AddAll(MenuItem)
	Return True
End Sub

'Rename a Title of an Item.
Public Sub RenameItem(oldTitle As String,newTitle As String) As Boolean
	If ListItem.IsInitialized = False Then Return False
	Dim c As Int
	For c = 0 To ListItem.Size-1 Step 3
		If ListItem.Get(c) = oldTitle Then
			ListItem.Set(c,newTitle)
			Return True
		End If
	Next
	Return False
End Sub

'Adds an item to the menu.
Public Sub AddMenuItem(Title As String,Image As Bitmap,EventName As String)
	ListItem.Add(Title)
	If Image.IsInitialized = False Then ListItem.Add(Null) Else ListItem.Add(Image)
	ListItem.Add(EventName)
End Sub

'Adds an item to the menu in the specified position.
Public Sub AddMenuItemAt(Title As String,Image As Bitmap,EventName As String,Position As Int)
	Position = Position*3
	ListItem.InsertAt(Position,EventName)
	If Image.IsInitialized = False Then ListItem.InsertAt(Position,Null) Else ListItem.InsertAt(Position,Image)
	ListItem.InsertAt(Position,Title)
End Sub

'Removes the item at the specified position.
Public Sub RemoveAt(Index As Int) As Boolean
	If MenuList.IsInitialized Then
		If Index < 0 OR Index > MenuList.Size-1 Then
			ToastMessageShow("Item not exist.",False)
			Return False	
		Else
			MenuList.RemoveAt(Index)
		End If
	End If
	If Index < 0 OR Index > (ListItem.Size/3)-1 Then
		ToastMessageShow("Item not exist.",False)
		Return False
	Else
		Dim c As Int
		For c = 0 To 2
			ListItem.RemoveAt(Index*3)
		Next
	End If
	Return True
End Sub

'Removes all item from to menu.
Public Sub RemoveAll As Boolean
	If MenuList.IsInitialized Then MenuList.Clear
	ListItem.Clear
	Return True
End Sub

'Returns the number of item storeds in the list [read only].
Public Sub Size As Int
	If ListItem.Size = 0 Then Return 0
	Return ListItem.Size/3+1
End Sub

'Programmatically opens the menu.
Public Sub OpenMenu
	ShowMenu
End Sub

'Locks the menu so that it can be opened until it is unlocked.
Public Sub MenuLock
	vMenuLock = True
End Sub

'Unlock the menu so that it can re-open if it was previously locked.
Public Sub MenuUnlock
	vMenuLock = False
End Sub

'Action to be sent to the class by listening to the menu button press event.
'
'Example:<code>
'Sub Activity_KeyPress (KeyCode As Int) As Boolean
'	If KeyCode = KeyCodes.KEYCODE_MENU Then Menu.KeyMenuPress
'End Sub</code>
Public Sub KeyMenuPress
	If vMenuLock OR AnimationInProgress Then Return
	If MenuIsShow Then CloseMenu Else ShowMenu
End Sub

Private Sub MenuList_ItemClick (Position As Int, Value As Object)
	vEventName = Value&"_Click"
	CloseMenu
End Sub

Private Sub ShowMenu
	If ListItem.Size = 0 Then
		ToastMessageShow("No item in the list.",False)
		Return
	End If
	'-------------------------
	pnlSfondo.Initialize("Sfondo")
	pnlSfondo.Color = Colors.Transparent
	pnlSfondo.Visible = False
	vActivity.AddView(pnlSfondo,0,0,100%x,100%y)
	pnlSfondo.BringToFront
	'-------------------------
	Dim c,Color As Int
	For c = nShade To 0 Step -1
		pnlShade(c).Initialize("")
		Color = Colors.ARGB(32/nShade,0,0,0)
		pnlShade(c).Background = SetRadialCorner(Color,Corner+c+1)
		If c = nShade Then
			pnlSfondo.AddView(pnlShade(nShade),0,0,0,0)
		Else
			pnlShade(nShade).AddView(pnlShade(c),0,0,0,0)
		End If 
	Next
	'-------------------------
	Dim TextSize As Int = Round(vHeightItem/Density/2.25)
	MenuList.Initialize("MenuList")
	MenuList.Color = ColorBackground
	MenuList.Background = SetRadialCorner(ColorBackground,Corner)
	MenuList.ScrollingBackgroundColor = ColorBackground
	MenuList.FastScrollEnabled = False
	MenuList.SingleLineLayout.ItemHeight = vHeightItem
    MenuList.SingleLineLayout.Label.TextSize = TextSize
    MenuList.SingleLineLayout.Label.TextColor = ColorText
    MenuList.SingleLineLayout.Label.Gravity = Gravity.CENTER_VERTICAL
	MenuList.TwoLinesAndBitmap.ItemHeight = MenuList.SingleLineLayout.ItemHeight
	MenuList.TwoLinesAndBitmap.Label.TextSize = MenuList.SingleLineLayout.Label.TextSize
	MenuList.TwoLinesAndBitmap.Label.TextColor = ColorText
	MenuList.TwoLinesAndBitmap.Label.Gravity = Gravity.CENTER_VERTICAL
	MenuList.TwoLinesAndBitmap.Label.Height = MenuList.SingleLineLayout.ItemHeight
	MenuList.TwoLinesAndBitmap.Label.Left = vHeightItem+6dip+EdgeIcon*2
	MenuList.TwoLinesAndBitmap.Label.Top = 0
	MenuList.TwoLinesAndBitmap.SecondLabel.Visible = False
	MenuList.TwoLinesAndBitmap.ImageView.Gravity = Gravity.FILL
	MenuList.TwoLinesAndBitmap.ImageView.SetLayout(6dip+EdgeIcon,EdgeIcon,vHeightItem-EdgeIcon*2,vHeightItem-EdgeIcon*2)
	RefillMenu
	'-------------------------
	pnlMenu.Initialize("")
	pnlMenu.Color = ColorBorder
	pnlMenu.Background = SetRadialCorner(ColorBorder,Corner)
	pnlShade(nShade).AddView(pnlMenu,0,0,0,0)
	pnlShade(nShade).AddView(MenuList,0,0,0,0)
	ResizeMenu
End Sub

Private Sub ResizeMenu
	Dim c,PosX,PosY,Height = HeightMenu As Int
	If vActivity.Width > vActivity.Height Then 'Landscape
		PosX = 100%x-WidthMenu-Dist
	Else 'Portrait
		PosX = 50%x-WidthMenu/2
	End If
	PosY = 100%y-Height+5
	pnlShade(nShade).SetLayout(PosX-nShade-1,PosY-nShade-1,WidthMenu+nShade*2,Height+nShade*2)
	For c = nShade To 1 Step -1
		pnlShade(c-1).SetLayout(nShade-c,nShade-c,WidthMenu+c*2,Height+c*2)
	Next
	pnlMenu.SetLayout(nShade,nShade,WidthMenu,Height)
	MenuList.SetLayout(nShade+1,nShade+1,WidthMenu-2,Height-2)
	pnlSfondo.Visible = True
	'-----------------------------------
	Dim Anim1,Anim2,Anim3 As AnimationPlus
	Dim AnimSet As AnimationSet
	Dim Translate As Int = 100%y-pnlShade(nShade).Top
	Anim1.InitializeAlpha("",0,1)
	Anim2.InitializeTranslate("AnimationShow",0,Translate,0,0)
'	Anim3.InitializeScaleCenter("",0.5,0.5,1,1,pnlShade(nShade))
	AnimSet.Initialize(False)
	AnimSet.AddAnimation(Anim1)
	AnimSet.AddAnimation(Anim2)
'	AnimSet.AddAnimation(Anim3)
	AnimSet.PersistAfter = False
	AnimSet.SetInterpolator(AnimSet.INTERPOLATOR_ACCELERATE_DECELERATE)
	AnimSet.Duration = AnimDuration
	AnimSet.Start(pnlShade(nShade))
	AnimationInProgress = True
End Sub

'Programmatically closes the menu.
Public Sub CloseMenu
	Dim Anim1,Anim2,Anim3 As AnimationPlus
	Dim AnimSet As AnimationSet
	Dim Translate As Int = 100%y-pnlShade(nShade).Top
	Anim1.InitializeAlpha("",1,0)
	Anim2.InitializeTranslate("AnimationHide",0,0,0,Translate)
'	Anim3.InitializeScaleCenter("",1,1,0.5,0.5,pnlShade(nShade))
	AnimSet.Initialize(False)
	AnimSet.AddAnimation(Anim1)
	AnimSet.AddAnimation(Anim2)
'	AnimSet.AddAnimation(Anim3)
	AnimSet.PersistAfter = False
	AnimSet.SetInterpolator(AnimSet.INTERPOLATOR_ACCELERATE_DECELERATE)
	AnimSet.Duration = AnimDuration
	AnimSet.Start(pnlShade(nShade))
	AnimationInProgress = True
End Sub

Private Sub AnimationHide_AnimationEnd
'	pnlSfondo.Visible = False
	AnimationInProgress = False
	MenuIsShow = False
'	pnlSfondo.RemoveAllViews '***
	pnlSfondo.RemoveView '***
	If vEventName <>"" Then
		If SubExists(vModule,vEventName) Then
			CallSub(vModule,vEventName)
			vEventName = ""
		End If
	End If
End Sub

Private Sub AnimationShow_AnimationEnd
	AnimationInProgress = False
	MenuIsShow = True
End Sub

Private Sub RefillMenu
	Dim c As Int
	Dim Title,EventName As String
	If ListItem.Size > 0 Then
		For c = 0 To ListItem.Size/3-1
			Title = ListItem.Get(0+c*3)
			Dim Image As Bitmap
			If ListItem.Get(1+c*3) <> Null Then
				Image.Initialize3(ListItem.Get(1+c*3))
			End If
			EventName = ListItem.Get(2+c*3)
			If Image.IsInitialized = False Then
				MenuList.AddSingleLine2(Title,EventName)
			Else
				MenuList.AddTwoLinesAndBitmap2(Title,"",Image,EventName)
			End If
		Next
	End If
End Sub

Private Sub HeightMenu As Int
	Dim Height,Diff = 1.5 As Float
	Dim Qty As Int
	Height = MenuList.SingleLineLayout.ItemHeight+Diff
	Qty = Min(ListItem.Size/3,vMaxItemShow)
	If Qty > (100%y-Dist/2)/Height Then Qty = (100%y-Dist/2)/Height
	Return Height*Qty+5-Diff
End Sub

Private Sub SetRadialCorner (Color As Int,Radius As Int) As GradientDrawable
	Dim grdDwb As GradientDrawable
	Dim Colore(2) As Int
	Colore(0) = Color
	Colore(1) = Colore(0)
	grdDwb.Initialize("TOP_BOTTOM",Colore)
	grdDwb.CornerRadius = Radius
	Return grdDwb
End Sub

Private Sub Sfondo_Click
	CloseMenu
End Sub
